#include "Adafruit_Sensor.h"
#include "Adafruit_AM2320.h"
#include "DHT.h"

#include <SD.h>   //SD Card Bibliothek
#include <SPI.h>  //Bibliothek zum zugriff auf die SPI Schnittstelle des Arduinos

File outFile; // Die Datei welche geschrieben bzw. gelesen werden soll
int pinCS = 10; // Chip Select Pin 10 auf dem Arduino Uno
String filename = "test.csv"; //Dateiname der zu lesenden bzw. schreibenden Datei

#define DHT11PIN 9
#define DHT22PIN 6

DHT dht11(DHT11PIN, DHT11);
DHT dht22(DHT22PIN, DHT22);

Adafruit_AM2320 am2320 = Adafruit_AM2320();

void setup() {
  Serial.begin(9600);

  while (!Serial) {
    delay(10); // hang out until serial port opens
  }

  pinMode(pinCS, OUTPUT);
  if (SD.begin()) { //Wenn die SD Card initialisiert werden konnte dann....
    Serial.println("SD Karte konnte erfolgreich geladen werden!"); //Ausgabe des Textes auf der Seriellen Schnittstelle
  } else {
    //Dieser Block wird ausgeführt wenn die SD Card nicht initialisiert werden konnte.
    Serial.println("SD Karte konnte NICHT erfolgreich geladen werden!"); //Ausgabe des Textes auf der Seriellen Schnittstelle
    Serial.println("---Programm wird beendet!"); //Ausgabe des Textes auf der Seriellen Schnittstelle
    return; //vorzeitiges beenden der setup Methode
  }

  dht11.begin();
  dht22.begin();

  Serial.println("Vergleich AM2320, DHT11 und DHT22");
  am2320.begin();

  Serial.println("Sensor\t| Temperatur\t|rel. Luftfeuchtigkeit");
  Serial.println("---------------------------------------------------");
}

void loop() {
  printDht11Data();
  printDht22Data();
  printAM2320Data();
  delay(2000);
}

void printDht11Data() {
  float temp = dht11.readTemperature();
  float humidity = dht11.readHumidity();
  printData("DHT11", temp, humidity);
}

void printDht22Data() {
  float temp = dht22.readTemperature();
  float humidity = dht22.readHumidity();
  printData("DHT22", temp, humidity);
}

void printAM2320Data() {
  float temp = am2320.readTemperature();
  float humidity = am2320.readHumidity();
  printData("AM2320", temp, humidity);
}

void printData(String sensor, float temp, float humidity) {
  Serial.println(sensor + "\t|" + String(temp, 2) + "°C\t|" + String(humidity, 2) + "%");
  writeContent(sensor, temp, humidity);
}

void writeContent(String sensor, float temp, float humidity) {
  outFile = SD.open(filename, FILE_WRITE); //Öffnet bzw. erzeugt die Datei im Modus schreibend
  if (outFile) { //Wenn die Datei existiert dann...
    outFile.println(sensor + ";" + String(temp, 2) + ";" + String(humidity, 2)); //Schreiben der Zeile in die Textdatei
    outFile.close(); //Schließen der Datei (Dieses ist wichtig da sonst beim beenden des Sketches dies Daten verloren gehen können.)
  } else {
    //Dieser Block wird ausgeführt wenn die Datei nicht erzeugt werden konnte.
    Serial.println("Fehler beim öffnen der Datei test.txt"); //Ausgabe des Textes auf der Seriellen Schnittstelle
  }
}
